"""
Portions of this module are derived from the open-source project
'blackboxprotobuf' (https://github.com/nccgroup/blackboxprotobuf),
originally developed by NCC Group Plc and distributed under the MIT License.

The source has been refactored and simplified for internal use within
scrapy_cffi, keeping only the commonly used APIs: `encode_message` and
`decode_message`.

MIT License (c) 2018–2023 NCC Group Plc
"""
from .api import *