class BloomFilter:
    def __init__(self, size: int, expected: int, hash_count: int = 0): ...
    def add(self, data: str | bytes): ...
    def exists(self, data: str | bytes) -> bool: ...
    def free(self): ...
    def clear(self): ...
    def estimated_false_positive_rate(self) -> float: ...
    def get_hash_count(self) -> int: ...
    def get_indices(self, data: str | bytes) -> list[int]: ...